from django.test import TestCase
from testapp.models import TestModel
from pmd_django.query_helpers import build_filtered_queryset


class TestBuildFilteredQueryset(TestCase):
    @classmethod
    def setUpTestData(cls):
        TestModel.objects.bulk_create([
            TestModel(status="active"),
            TestModel(status="inactive"),
            TestModel(status="active"),
            TestModel(status="pending"),
        ])

    def test_filters_by_search(self):
        qs = TestModel.objects.all()
        payload = {
            "search": "act",
        }
        filtered = build_filtered_queryset(
            qs,
            payload,
            extra_filter_from_request={"number": "status__icontains"},
        )
        self.assertTrue(all("act" in obj.status for obj in filtered))

    def test_stage_filter_applied(self):
        qs = TestModel.objects.all()
        payload = {
            "stageFilter": "pending"
        }
        filtered = build_filtered_queryset(qs, payload, field="status", counted_values=["active", "inactive", "pending"])
        self.assertEqual(list(filtered.values_list("status", flat=True)), ["pending"])

    def test_user_filters_applied(self):
        qs = TestModel.objects.all()
        payload = {
            "userFilters": [
                {"field": "status", "condition": "exact", "value": "active"},
            ]
        }
        filtered = build_filtered_queryset(qs, payload)
        statuses = list(filtered.values_list("status", flat=True))
        self.assertEqual(statuses, ["active", "active"])



import io
import json
from datetime import datetime, timezone

from django.db.models import Count
from django.http import FileResponse, JsonResponse
from django.test import RequestFactory, TestCase
from django.utils.timezone import now
from openpyxl.reader.excel import load_workbook

from pmd_django import GenericTableView
from pmd_django.generic_table.generic_table import (
    _apply_filters,
    _apply_ordering,
    _apply_user_filters,
    _build_counts,
    _generate_download_response,
    _get_allowed_fields,
    _get_page_size,
    _paginate_queryset,
    view,
)
from testapp.models import TestRelatedModel, TestModel

class TestGenericTable(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.factory = RequestFactory()

        tm = TestModel.objects.create(status="active")

        TestModel.objects.bulk_create(
            [
                TestModel(status="inactive"),
                TestModel(status="active"),
                TestModel(status="inactive"),
                TestModel(status="active"),
            ]
        )

        TestRelatedModel.objects.create(name="Related Item", related=tm)

    def test_build_counts(self):
        counted_values = ["active", "inactive", "pending"]
        queryset = TestModel.objects.all()
        result = _build_counts(queryset, "status", counted_values)

        self.assertEqual(result, {"active": 3, "inactive": 2, "pending": 0, "ALL": 5})

    def test_get_page_size_valid(self):
        request = self.factory.get("/?page_size=20")
        self.assertEqual(_get_page_size(request, None), 20)

    def test_get_page_size_invalid(self):
        request = self.factory.get("/?page_size=invalid")
        self.assertEqual(_get_page_size(request, None), 30)

    def test_apply_filters(self):
        request = self.factory.get("/?status=active")
        extra_filters = {}
        extra_filter_from_request = {"status": "status__iexact"}

        filtered_qs = _apply_filters(
            TestModel.objects.all(), request, extra_filters, extra_filter_from_request
        )
        self.assertTrue(filtered_qs.exists())

    def test_apply_ordering(self):
        request = self.factory.get("/?sort_by=id&sort_order=desc")
        ordered_qs = _apply_ordering(TestModel.objects.all(), request)
        ids = list(ordered_qs.values_list("id", flat=True))
        self.assertEqual(ids, sorted(ids, reverse=True))

    def test_paginate_queryset(self):
        request = self.factory.get("/?page=2&page_size=2")
        queryset = TestModel.objects.all().order_by("id")

        page_obj, paginator = _paginate_queryset(queryset, request, 2)

        self.assertEqual(page_obj.number, 2)
        self.assertEqual(paginator.num_pages, 3)

        all_ids = list(TestModel.objects.all().order_by("id").values_list("id", flat=True))
        expected_ids = all_ids[2:4]  # page 2, page_size 2
        actual_ids = list(page_obj.object_list.values_list("id", flat=True))

        self.assertEqual(actual_ids, expected_ids)

    def test_view(self):
        request = self.factory.get("/?page=1&sort_by=id")
        queryset = TestModel.objects.all()

        response = view(queryset, request, "status", ["active", "inactive"])

        self.assertIsInstance(response, JsonResponse)
        response_data = json.loads(response.content)

        self.assertIn("pagination", response_data)
        self.assertIn("stageCounts", response_data)

    def test_valid_user_filters(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [
                        {"field": "status", "condition": "exact", "value": "active"},
                        {
                            "field": "id",
                            "condition": "gte",
                            "value": 3,
                            "logicalOperator": "AND",
                        },
                    ]
                )
            },
        )
        queryset = _apply_user_filters(TestModel.objects.all(), request)
        self.assertTrue(queryset.exists())

    def test_invalid_field_filtering(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [{"field": "password", "condition": "exact", "value": "test"}]
                )
            },
        )
        with self.assertRaises(ValueError) as e:
            _apply_user_filters(TestModel.objects.all(), request)
        self.assertEqual(str(e.exception), "Filtering on 'password' is not allowed")

    def test_invalid_condition_filtering(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [{"field": "status", "condition": "hacked", "value": "active"}]
                )
            },
        )
        with self.assertRaises(ValueError) as e:
            _apply_user_filters(TestModel.objects.all(), request)
        self.assertEqual(str(e.exception), "Condition 'hacked' is not supported")

    def test_json_parsing_error(self):
        request = self.factory.get("/", data={"user_filters": "[INVALID JSON"})
        with self.assertRaises(ValueError) as e:
            _apply_user_filters(TestModel.objects.all(), request)
        self.assertEqual(str(e.exception), "Invalid JSON in user_filters")

    def test_missing_value_filter(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [{"field": "status", "condition": "exact", "value": None}]
                )
            },
        )
        queryset = _apply_user_filters(TestModel.objects.all(), request)
        self.assertEqual(queryset.count(), 5)  # Should return full dataset

    def test_apply_user_filters_where_we_have_dangling_operator(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [
                        {
                            "field": "id",
                            "condition": "gte",
                            "value": 3,
                            "logicalOperator": "AND",
                        }
                    ]
                )
            },
        )

        filtered_qs = _apply_user_filters(TestModel.objects.all(), request)
        self.assertTrue(filtered_qs.exists())

    def test_sql_injection_attempt(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [
                        {
                            "field": "status",
                            "condition": "exact",
                            "value": "'; DROP TABLE users; --",
                        }
                    ]
                )
            },
        )
        queryset = _apply_user_filters(TestModel.objects.all(), request)
        self.assertEqual(queryset.count(), 0)  # Should not execute harmful queries

    def test_unmatched_operators_and_filters(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [
                        {"field": "status", "condition": "exact", "value": "active"},
                        {
                            "field": "id",
                            "condition": "gte",
                            "value": 3,
                            "logicalOperator": "AND",
                        },
                        {
                            "field": "status",
                            "condition": "icontains",
                            "value": "inactive",
                        },
                        # No logicalOperator for this one
                    ]
                )
            },
        )
        with self.assertRaises(ValueError) as e:
            _apply_user_filters(TestModel.objects.all(), request)
        self.assertIn(
            "Filter-operator mismatch: expected 2 operators, got 1", str(e.exception)
        )

    def test_standard_model_fields(self):
        qs = TestModel.objects.all()
        allowed_fields = _get_allowed_fields(qs)

        self.assertIn("id", allowed_fields)
        self.assertIn("status", allowed_fields)

    def test_related_model_fields(self):
        qs = TestModel.objects.all()
        allowed_fields = _get_allowed_fields(qs)

        self.assertIn("testrelatedmodel__name", allowed_fields)

    def test_annotated_fields(self):
        qs = TestModel.objects.annotate(status_count=Count("status"))
        allowed_fields = _get_allowed_fields(qs)

        self.assertIn("status_count", allowed_fields)

    def test_values_fields(self):
        qs = TestModel.objects.values("id", "status")
        qs.extra_allowed_fields = {"id", "status"}

        allowed_fields = _get_allowed_fields(qs)

        self.assertIn("id", allowed_fields)
        self.assertIn("status", allowed_fields)
        self.assertNotIn("nope", allowed_fields)

    def test_rejects_unauthorized_fields(self):
        qs = TestModel.objects.all()
        allowed_fields = _get_allowed_fields(qs)

        self.assertNotIn("password_hash", allowed_fields)  # Doesn't exist in the model
        self.assertNotIn("random_field", allowed_fields)  # Should not be allowed

    def test_apply_user_filters_with_annotated_field(self):
        qs = TestModel.objects.annotate(status_count=Count("status"))

        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [{"field": "status_count", "condition": "gte", "value": 1}]
                )
            },
        )
        filtered_qs = _apply_user_filters(qs, request)
        self.assertTrue(filtered_qs.exists())

    def test_apply_user_filters_with_related_field(self):
        request = self.factory.get(
            "/",
            data={
                "user_filters": json.dumps(
                    [
                        {
                            "field": "testrelatedmodel__name",
                            "condition": "exact",
                            "value": "Related Item",
                        }
                    ]
                )
            },
        )
        qs = _apply_user_filters(TestModel.objects.all(), request)
        self.assertTrue(qs.exists())

    def test_generate_excel_response(self):
        queryset = TestModel.objects.all()
        values_list = ["status"]

        response = _generate_download_response(queryset, values_list)
        worksheet = self._load_excel_from_response(response)

        expected_headers = ["status"]
        headers = [cell.value for cell in worksheet[1]]
        self.assertEqual(headers, expected_headers)

        data_rows = list(worksheet.iter_rows(min_row=2, values_only=True))
        expected_data = [
            ("active",),
            ("inactive",),
            ("active",),
            ("inactive",),
            ("active",),
        ]
        self.assertEqual(data_rows, expected_data)

    def test_generate_excel_response_with_list_field(self):
        TestModel.objects.create(status="tagged", tags=["TEST", "ALPHA"])

        queryset = TestModel.objects.filter(status="tagged")
        values_list = ["status", "tags"]

        response = _generate_download_response(queryset, values_list)
        worksheet = self._load_excel_from_response(response)

        headers = [cell.value for cell in worksheet[1]]
        self.assertEqual(headers, ["status", "tags"])

        data_row = list(worksheet.iter_rows(min_row=2, max_row=2, values_only=True))[0]
        self.assertEqual(data_row[0], "tagged")
        self.assertIsInstance(data_row[1], str)
        self.assertIn("TEST", data_row[1])
        self.assertIn("ALPHA", data_row[1])

    def test_generate_excel_response_with_timezone_datetime(self):
        aware_datetime = now().astimezone(timezone.utc)
        TestModel.objects.create(status="datetime", created_at=aware_datetime)

        queryset = TestModel.objects.filter(status="datetime")
        values_list = ["status", "created_at"]

        response = _generate_download_response(queryset, values_list)
        worksheet = self._load_excel_from_response(response)

        headers = [cell.value for cell in worksheet[1]]
        self.assertEqual(headers, ["status", "created_at"])

        data_row = list(worksheet.iter_rows(min_row=2, max_row=2, values_only=True))[0]
        self.assertEqual(data_row[0], "datetime")
        self.assertIsInstance(data_row[1], datetime)
        self.assertIsNone(data_row[1].tzinfo, "Datetime should be naive (tzinfo=None)")

    def test_json_and_excel_field_handling(self):
        class DummyView(GenericTableView):
            model = TestModel
            stage_field = "status"
            table_headers = {
                "id": None,
                "status": ("Status", ("default", {})),
                "created_at": None,
            }

            @property
            def generic_table_view_kwargs(self):
                return {
                    "field": self.stage_field,
                    "values_list": list(self.table_headers.keys()),
                    "download_fields": [k for k, v in self.table_headers.items() if v is not None or k == "id"],
                    "counted_values": [],
                    "data_key": "data",
                    "final_json_hook": self.final_json_hook,
                }

            def final_json_hook(self, data):
                data["displayHeaders"] = {k: v[0] for k, v in self.table_headers.items() if v}
                data["filterFieldTypes"] = {
                    k: v[1][0] for k, v in self.table_headers.items() if v
                }
                return data

        # Excel export should only include 'id' and 'status'
        excel_request = self.factory.get("/", HTTP_ACCEPT="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
        view_instance = DummyView()
        excel_response = view_instance.get(excel_request)
        worksheet = self._load_excel_from_response(excel_response)
        headers = [cell.value for cell in worksheet[1]]
        self.assertEqual(headers, ["id", "status"])

        # JSON export should include all table_headers keys (even those with None)
        json_request = self.factory.get("/")
        json_response = view_instance.get(json_request)
        data = json.loads(json_response.content)["data"][0]
        self.assertIn("id", data)
        self.assertIn("status", data)
        self.assertIn("created_at", data)

    def _load_excel_from_response(self, response: FileResponse):
        self.assertIsInstance(response, FileResponse)
        self.assertEqual(
            response["Content-Type"],
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        )
        output = io.BytesIO()
        for chunk in response.streaming_content:
            output.write(chunk)
        output.seek(0)
        workbook = load_workbook(output)
        return workbook.active



import json

from django.test import TestCase, RequestFactory
from django.views.decorators.http import require_POST
from testapp.models import TestModel, TestRelatedModel, UniqueModel
from pmd_django.api import api


@require_POST
@api({
    "code": {"type": "string", "required": True, "empty": False},
})
def create_unique_view(request):
    related = TestRelatedModel.objects.first()
    return UniqueModel.objects.create(code=request.json["code"], related=related)


class TestJsonDecoratorWithRelations(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.factory = RequestFactory()

        test_model = TestModel.objects.create(status="linked")
        cls.related = TestRelatedModel.objects.create(name="Related Obj", related=test_model)


    def test_nested_relationships_are_serialized(self):
        request = self.factory.post(
            "/",
            data=json.dumps({"code": "XYZ"}),
            content_type="application/json",
        )
        response = create_unique_view(request)

        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)

        self.assertIn("code", data)
        self.assertEqual(data["code"], "XYZ")

        self.assertIn("related", data)
        self.assertIsInstance(data["related"], dict)
        self.assertEqual(data["related"]["name"], "Related Obj")

        self.assertIn("related", data["related"])
        self.assertEqual(data["related"]["related"]["status"], "linked")


    def test_rejects_fields_not_in_schema(self):
        request = self.factory.post(
            "/",
            data=json.dumps({
                "code": "SHOULD_FAIL",
                "id": 999,
                "unvalidated_field": "should be rejected"
            }),
            content_type="application/json",
        )

        response = create_unique_view(request)
        self.assertEqual(response.status_code, 400)

        data = json.loads(response.content)

        self.assertIn("errors", data)
        self.assertIn("id", data["errors"])
        self.assertIn("unvalidated_field", data["errors"])


    def test_rejects_empty_string_by_default_for_required_string(self):
        request = self.factory.post(
            "/",
            data=json.dumps({"code": ""}),
            content_type="application/json",
        )
        response = create_unique_view(request)

        self.assertEqual(response.status_code, 400)

        data = json.loads(response.content)
        self.assertIn("errors", data)
        self.assertIn("code", data["errors"])
        self.assertEqual(data["errors"]["code"], ["empty values not allowed"])


import json
from django.http import QueryDict
from pmd_django.generic_table.generic_table import apply_all_filters


def build_filtered_queryset(
        qs,
        payload: dict,
        *,
        field=None,
        counted_values=None,
        extra_filters=None,
        extra_filter_from_request=None,
):
    fake_get = QueryDict("", mutable=True)

    if payload.get("search"):
        fake_get["number"] = payload["search"]
    if payload.get("sortBy"):
        fake_get["sort_by"] = payload["sortBy"]
    if payload.get("sortOrder"):
        fake_get["sort_order"] = payload["sortOrder"]
    if payload.get("stageFilter"):
        fake_get["filter"] = payload["stageFilter"]
    if payload.get("userFilters"):
        fake_get["user_filters"] = json.dumps(payload["userFilters"])

    class FakeRequest:
        GET = fake_get
        headers = {}

    filtered_qs, _ = apply_all_filters(
        qs,
        FakeRequest(),
        field=field,
        counted_values=counted_values,
        extra_filters=extra_filters,
        extra_filter_from_request=extra_filter_from_request,
    )

    return filtered_qs



import json
import re
from functools import wraps
from django.forms.models import model_to_dict
from django.http import JsonResponse, HttpResponse
from django.db import IntegrityError
from django.db.models import Model
from cerberus import Validator


def _model_to_dict_recursive(instance, visited=None):
    if not isinstance(instance, Model):
        return instance

    if visited is None:
        visited = set()

    key = (instance.__class__, instance.pk)
    if key in visited:
        return instance.pk

    visited.add(key)
    data = model_to_dict(instance)

    for field in instance._meta.get_fields():
        if field.is_relation and field.concrete and not field.many_to_many:
            related = getattr(instance, field.name, None)
            if related is not None:
                data[field.name] = _model_to_dict_recursive(related, visited)

    return data


def _apply_required_string_defaults(rules: dict) -> dict:
    for key, rule in rules.items():
        if (
                rule.get("type") == "string"
                and rule.get("required") is True
                and "empty" not in rule
        ):
            rule["empty"] = False
    return rules


def _parse_json_body(request):
    try:
        return json.loads(request.body), None
    except json.JSONDecodeError:
        return None, JsonResponse(
            {"errors": {"non_field_errors": ["Invalid JSON"]}}, status=400
        )


def _validate_and_clean_data(data, rules: dict):
    rules = _apply_required_string_defaults(rules)
    validator = Validator(rules)
    if not validator.validate(data):
        return None, JsonResponse({"errors": validator.errors}, status=400)
    cleaned = {k: validator.document[k] for k in rules if k in validator.document}
    return cleaned, None


def _handle_integrity_error(error: IntegrityError):
    match = re.search(r"Key \((\w+)\)=\(.+?\) already exists", str(error))
    if match:
        return JsonResponse({"errors": {match.group(1): ["already exists"]}}, status=400)
    return JsonResponse({"errors": {"non_field_errors": ["Integrity error"]}}, status=400)


def _serialize_response(result, status=200):
    if isinstance(result, HttpResponse):
        return result
    if isinstance(result, tuple):
        result, status = result
    if isinstance(result, Model):
        return JsonResponse(_model_to_dict_recursive(result), status=status)
    if isinstance(result, list) and result and isinstance(result[0], Model):
        return JsonResponse(
            [_model_to_dict_recursive(obj) for obj in result],
            safe=False,
            status=status
        )
    return JsonResponse(result, status=status)


def api(rules: dict | None = None):
    def decorator(view_func):
        @wraps(view_func)
        def _wrapped_view(request, *args, **kwargs):
            data, error_response = _parse_json_body(request)
            if error_response:
                return error_response

            if rules:
                data, error_response = _validate_and_clean_data(data, rules)
                if error_response:
                    return error_response

            request.json = data

            try:
                result = view_func(request, *args, **kwargs)
            except IntegrityError as e:
                return _handle_integrity_error(e)

            return _serialize_response(result)

        return _wrapped_view
    return decorator



import unittest
from django.urls import path, include

from pmd_django.management.commands.urls import route_to_key, route_to_value, extract_routes


def dummy_view(request): pass


class TestGeneratedUrlsHelpers(unittest.TestCase):
    def test_route_to_key_static(self):
        self.assertEqual(route_to_key("/dashboard/get_permits"), "DASHBOARD_GET_PERMITS")
        self.assertEqual(route_to_key("/pm-tracker/bulk_update_tags"), "PM_TRACKER_BULK_UPDATE_TAGS")
        self.assertEqual(route_to_key(""), "ROOT")
        self.assertEqual(route_to_key("/permit/<int:permit_id>/workflow"), "PERMIT_PERMIT_ID_WORKFLOW")

    def test_route_to_key_weird_chars(self):
        self.assertEqual(route_to_key("/crazy-Route/<uuid:abc>/yo!"), "CRAZY_ROUTE_ABC_YO")

    def test_route_to_value_static(self):
        self.assertEqual(
            route_to_value("/dashboard/get_permits"),
            '"/dashboard/get_permits"'
        )

    def test_route_to_value_dynamic_single(self):
        self.assertEqual(
            route_to_value("/permit/<int:permit_id>/workflow"),
            "(permit_id: string | number) => `/permit/${permit_id}/workflow`"
        )

    def test_route_to_value_dynamic_multi(self):
        self.assertEqual(
            route_to_value("/a/<int:x>/b/<slug:y>/c"),
            "(x: string | number, y: string | number) => `/a/${x}/b/${y}/c`"
        )

    def test_extract_routes_simple(self):
        urlpatterns = [
            path("dashboard/get_permits", dummy_view),
            path("permit/<int:permit_id>/workflow", dummy_view),
        ]
        result = extract_routes(urlpatterns)
        self.assertIn("dashboard/get_permits", result)
        self.assertIn("permit/<int:permit_id>/workflow", result)

    def test_extract_routes_nested(self):
        urlpatterns = [
            path("outer/", include([
                path("inner/", dummy_view),
                path("deep/<int:item_id>/view", dummy_view),
            ])),
        ]
        result = extract_routes(urlpatterns)
        self.assertIn("outer/inner/", result)
        self.assertIn("outer/deep/<int:item_id>/view", result)



import os
import re
from django.core.management.base import BaseCommand
from django.urls import get_resolver, URLPattern, URLResolver
from django.urls.resolvers import RoutePattern


def extract_routes(urlpatterns, prefix=""):
    routes = []

    for entry in urlpatterns:
        if isinstance(entry, URLPattern) and isinstance(entry.pattern, RoutePattern):
            route = prefix + str(entry.pattern)
            routes.append(route)
        elif isinstance(entry, URLResolver):
            nested_prefix = prefix + str(entry.pattern)
            routes += extract_routes(entry.url_patterns, nested_prefix)

    return routes


def route_to_key(route: str) -> str:
    route = route.strip("/")
    if not route:
        return "ROOT"

    # Convert path params like <int:id> to just "id"
    route = re.sub(r"<[^>:]+:([^>]+)>", r"\1", route)
    route = re.sub(r"<([^>]+)>", r"\1", route)

    # Normalize: replace `/` and `-` with `_`
    key = route.upper().replace("/", "_").replace("-", "_")

    # Strip invalid characters (just in case)
    key = re.sub(r"[^A-Z0-9_]", "", key)

    # Ensure it starts with a letter or underscore
    if not re.match(r"^[A-Z_]", key):
        key = "_" + key

    return key



def route_to_value(route: str) -> str:
    params = re.findall(r"<(?:[^>:]+:)?([^>]+)>", route)
    if not params:
        return f'"{route}"'

    param_str = ", ".join(f"{p}: string | number" for p in params)
    path_str = re.sub(r"<(?:[^>:]+:)?([^>]+)>", r"${\1}", route)
    return f"({param_str}) => `{path_str}`"


class Command(BaseCommand):
    help = "Export Django URL patterns as a TypeScript file"

    def add_arguments(self, parser):
        parser.add_argument(
            "--output",
            default="frontend/src/urls.ts",
            help="Path to output TypeScript file",
        )

    def handle(self, *args, **options):
        resolver = get_resolver()
        all_routes = extract_routes(resolver.url_patterns)

        filtered_routes = [
            r for r in all_routes
            if not r.startswith("/admin")
               and not r.startswith("static/")
               and not r.startswith("__debug__")
        ]

        output_lines = [
            "// NO EDIT: Generated via `python manage.py urls`\n\n",
            "export const URLS = {\n"
        ]
        for route in sorted(filtered_routes):
            key = route_to_key(route)
            value = route_to_value(route)
            output_lines.append(f"  {key}: {value},\n")
        output_lines.append("};\n\nexport type URLS = typeof URLS;\n")

        output_path = options["output"]
        os.makedirs(os.path.dirname(output_path), exist_ok=True)
        with open(output_path, "w") as f:
            f.writelines(output_lines)

        self.stdout.write(self.style.SUCCESS(f"Exported {len(filtered_routes)} routes to {output_path}"))



from django.db import models
from django.utils.timezone import now


class TestModel(models.Model):
    status = models.CharField(max_length=100)
    tags = models.JSONField(default=list)
    created_at = models.DateTimeField(default=now)
    metadata_not_for_download = models.TextField()

    class Meta:
        db_table = "test_model"
        app_label = 'testapp'

class TestRelatedModel(models.Model):
    name = models.TextField()
    related = models.ForeignKey(TestModel, on_delete=models.CASCADE)

    class Meta:
        db_table = "test_related_model"
        app_label = 'testapp'

class UniqueModel(models.Model):
    code = models.TextField(unique=True)
    related = models.ForeignKey("testapp.TestRelatedModel", on_delete=models.CASCADE, null=True)

    class Meta:
        db_table = "test_unique_model"
        app_label = 'testapp'
